// SerialMon, firmware for serial communications monitor
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License 
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "Common.h"
#include "Serial.h"
#include <stdlib.h>


byte UART_TxBuf[UART_TX_BUFFER_SIZE];
volatile byte UART_TxHead;
volatile byte UART_TxTail; 
bool bOverflow;

byte UART_RxBuf[UART_RX_BUFFER_SIZE]; 
volatile byte UART_RxHead;
volatile byte UART_RxTail;


#ifdef USE_UART1
	#define UDR		UDR1
	#define UCSRA	UCSR1A
	#define UCSRB	UCSR1B
	#define UCSRC	UCSR1C
	#define UBRRH	UBRR1H
	#define UBRRL	UBRR1L
	#define SIG_UART_DATA	SIG_UART1_DATA
	#define SIG_UART_RECV	SIG_UART1_RECV
#endif

#if MCU == at90s4433
    #if !defined(UBRRL)
        #define UBRRL   UBRR
    #endif
#endif

SIGNAL (SIG_USART_DATA)	// UART Data ready interrupt
{ 
	byte tmptail;

	/* Check if all data is transmitted */
	if ( UART_TxHead != UART_TxTail )
	{
		/* Calculate buffer index */
		tmptail = ( UART_TxTail + 1 ) & UART_TX_BUFFER_MASK;
		UART_TxTail = tmptail;      /* Store new index */
	
		UDR0 = UART_TxBuf[tmptail];  /* Start transmition */
	}
	else
	{
		UCSR0B &= ~(1<<UDRIE0);         /* Disable UDRE interrupt */
	}
}

SIGNAL (SIG_USART_RECV)	// UART Data ready interrupt
{ 
	byte data;
	byte tmphead;

	data = UDR0;                 /* Read the received data */
	/* Calculate buffer index */
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
	UART_RxHead = tmphead;      /* Store new index */ 

	if ( tmphead == UART_RxTail )
	{
		/* ERROR! Receive buffer overflow */
	}
	
	UART_RxBuf[tmphead] = data; /* Store received data in buffer */
}


void InitRS232( unsigned short nBaud )
{
	UART_TxTail = 0;
	UART_TxHead = 0;

	UART_RxTail = 0;
	UART_RxHead = 0;

	UBRR0H = nBaud >> 8;
	UBRR0L = nBaud & 0xFF;

	UCSR0A = 0;
	UCSR0B = 0;

	UCSR0C = _BV(UCSZ01) | _BV(UCSZ00);	// 8,n,1

	UCSR0B |=  _BV(TXEN0);
	UCSR0B |=  _BV(RXCIE0) | _BV(RXEN0);

	bOverflow = false;
}



void RS232Send( char *str )
{
	while ( *str != 0 )
	{
		RS232SendChar( *str );
		str++;
	}
}

void RS232Send_P( PGM_P s  )
{
	char b;
	goto ReadByte;
	while ( b != 0 )
	{
		RS232SendChar(b );
		s++;
ReadByte:
		b = pgm_read_byte( s );
	}
}


void RS232SendInt( int v )
{
	char s[7];
	itoa( v, s, 10 );
	RS232Send( s );
}

void RS232SendLong( long v )
{
	char s[11];
	ltoa( v, s, 10 );
	RS232Send( s );
}

void RS232SendHex( byte v )
{
	byte b = v >> 4;
	if ( b < 10 )
		RS232SendChar( '0' + b );
	else
		RS232SendChar( 'A' + b - 10 );
	b = v & 0xF;
	if ( b < 10 )
		RS232SendChar( '0' + b );
	else
		RS232SendChar( 'A' + b - 10 );
}

void RS232SendCRLF( void )
{
	RS232SendChar( '\r' );
	RS232SendChar( '\n' );
}


byte RS232ReadByte( void )
{
	byte tmptail;
	
	while ( UART_RxHead == UART_RxTail )  /* Wait for incomming data */
		;
	tmptail = ( UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;/* Calculate buffer index */
	
	UART_RxTail = tmptail;                /* Store new index */
	
	return UART_RxBuf[tmptail];           /* Return data */
}

bool RS232ReadDataAvailable( void )
{
	return ( UART_RxHead != UART_RxTail ); /* Return 0 (FALSE) if the receive buffer is empty */
}
