// SerialMon, firmware for serial communications monitor
// Copyright (C) 2008, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef __AVR_SERIAL_H__
#define __AVR_SERIAL_H__
 
enum EParity
{
    eNone,
    eOdd,
    eEven
} ;

extern void InitRS232( unsigned short nBaud );
extern void RS232SetParity( enum EParity eParity );
extern void RS232SetStopBits( byte nBits );
extern void RS232SetCharSize( byte nBits );

#ifdef RS232_TX
	extern void RS232Send( char *str );
	extern void RS232Send_P( PGM_P s  );
	extern void RS232SendInt( int v );
	extern void RS232SendLong( long v );
	extern void RS232SendHex( byte v );
	extern void RS232SendCRLF( void ); 
#endif
#ifdef RS232_RX
	extern byte RS232ReadByte( void );
	extern bool RS232ReadDataAvailable( void );
#endif

#ifndef UART_TX_BUFFER_SIZE
	#define UART_TX_BUFFER_SIZE 128		/* 2,4,8,16,32,64,128 or 256 bytes */ 
#endif
#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )
#if ( UART_TX_BUFFER_SIZE & UART_TX_BUFFER_MASK )
	#error TX buffer size is not a power of 2
#endif

#ifndef UART_RX_BUFFER_SIZE
	#define UART_RX_BUFFER_SIZE 128		/* 2,4,8,16,32,64,128 or 256 bytes */
#endif
#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
	#error RX buffer size is not a power of 2
#endif


extern byte UART_TxBuf[];
extern volatile byte UART_TxHead;
extern volatile byte UART_TxTail; 
extern bool bOverflow;

static inline void RS232SendChar( byte data )
{
	unsigned char tmphead;

	/* Calculate buffer index */
	tmphead = ( UART_TxHead + 1 ) & UART_TX_BUFFER_MASK; 
	// Never wait.
	if ( tmphead == UART_TxTail )
	{
		bOverflow = true;	// can't send data.  When we eventually have room, send an overflow error.
	}
	else
	{
		UART_TxBuf[tmphead] = data;           /* Store data in buffer */
		UART_TxHead = tmphead;                /* Store new index */

		UCSR0B |= (1<<UDRIE0);                    /* Enable UDRE interrupt */
	}
}


static inline void RS232SendChars( byte data1, byte data2 )
{
	unsigned char tmphead;

	/* Calculate buffer index */
	tmphead = ( UART_TxHead + 1 ) & UART_TX_BUFFER_MASK;
	// Never wait.
	if ( tmphead == UART_TxTail )
	{
		bOverflow = true;	// can't send data.  When we eventually have room, send an overflow error.
	}
	else
	{
		UART_TxBuf[tmphead] = data1;          /* Store data in buffer */
		tmphead = ( tmphead + 1 ) & UART_TX_BUFFER_MASK;
		UART_TxBuf[tmphead] = data2;        /* Store data in buffer */
		UART_TxHead = tmphead;                /* Store new index */

		UCSR0B |= (1<<UDRIE0);                /* Enable UDRE interrupt */
	}
}



#endif
