#include "Common.h"
#include <stdlib.h>
#include "single_wire_UART.h"   //UART settings and device spesific.

/* Counter values */
#define UART_STATE_IDLE       0
#define RECEIVE_FIRST_DATA    3
#define RECEIVE_LAST_DATA     17
#define RECEIVE_STOP_1        19
#define RECEIVE_FINISH        23
#define TRANSMIT_FIRST_DATA   2
#define TRANSMIT_LAST_DATA    16
#define TRANSMIT_STOP_1       18
#define TRANSMIT_STOP_2       20
#define TRANSMIT_FINISH       22

/* Prototypes */
void SW_UART_Complete_Transfer(void);
void SW_UART_Initiate_Transmission(void);

volatile uint8_t SW_UART_status;          
static volatile uint8_t UART_counter;     

/* Communication parameters. */
static volatile uint8_t   UART_Tx_data;     
static volatile uint8_t   UART_Rx_data;     
static volatile uint8_t   UART_Tx_buffer;   
static volatile uint8_t   UART_Rx_buffer;   


// TODO - don't use interrupts. At least make a version that doesn't

void SW_UART_Enable(void)
{
	SW_UART_DDR |= _BV(SW_UART_PIN_NUMBER);

	SET_UART_PIN();

	SW_UART_status = 0x00;
	UART_counter = UART_STATE_IDLE;
	CLEAR_UART_TIMER_ON_COMPARE_MATCH();

	//Set up interrupts
	//INITIALIZE_UART_EXTERNAL_INTERRUPT(); //Set up the external interrupt to wait for incoming start bit on correct pin.
	//CLEAR_UART_EXTERNAL_INTERRUPT_FLAG(); //Clear flag in case it is already set for some reason.
	//ENABLE_UART_EXTERNAL_INTERRUPT();
}


/*
void SW_UART_Disable()
{
SW_UART_status = 0x00;
DISABLE_UART_TIMER_INTERRUPT();
DISABLE_UART_EXTERNAL_INTERRUPT();
STOP_UART_TIMER();
}
*/


void SW_UART_Transmit(uint8_t data)
{
	SET_FLAG( SW_UART_status, SW_UART_TX_BUFFER_FULL );
	UART_Tx_buffer = data;

	//Start transmission if no ongoing communication.
	if( UART_counter == UART_STATE_IDLE )
	{
		//Copy byte from buffer and clear buffer full flag.
		UART_Tx_data = UART_Tx_buffer;
		CLEAR_FLAG( SW_UART_status, SW_UART_TX_BUFFER_FULL );
		UART_counter = TRANSMIT_FIRST_DATA;       //Update counter so the 1. data bit is the next bit to be transmitted.
		CLEAR_UART_TIMER();                       //Clear timer.
		CLEAR_UART_PIN();
		SET_UART_TIMER_COMPARE_START_TRANSMIT();  //Set timer compare value.
		CLEAR_UART_TIMER_INTERRUPT_FLAG();        //Make sure timer interrupt flag is not set.
		ENABLE_UART_TIMER_INTERRUPT();
		START_UART_TIMER();
	}
}




//#pragma vector=SW_UART_TIMER_COMPARE_INTERRUPT_VECTOR
//__interrupt void Timer_SW_UART_ISR(void)
SIGNAL (SIG_OUTPUT_COMPARE2A)
{
	SET_UART_TIMER_COMPARE_WAIT_ONE(); //Set timer compare value to trigger the ISR once every bit period.

	////Initiate transmission if there is data in TX_buffer. This is done in the
	////same way as in the UART_transmit routine.
	//if( READ_FLAG(SW_UART_status, SW_UART_TX_BUFFER_FULL) )
	//{
	//  UART_Tx_data = UART_Tx_buffer;
	//  CLEAR_FLAG( SW_UART_status, SW_UART_TX_BUFFER_FULL );
	//  DISABLE_UART_EXTERNAL_INTERRUPT();
	//  CLEAR_UART_PIN();                         //Write start bit.
	//  UART_counter = TRANSMIT_FIRST_DATA;       //Update counter so the 1. data bit is the next bit to be transmitted.
	//  STOP_UART_TIMER();                        //Stop timer to reset prescaler.
	//  CLEAR_UART_TIMER();                       //Clear timer.
	//  SET_UART_TIMER_COMPARE_START_TRANSMIT();  //Set timer compare value
	//  CLEAR_UART_TIMER_INTERRUPT_FLAG();        //Make sure timer interrupt flag is not set.
	//  START_UART_TIMER();
	//  return;                                   //Exit ISR so the counter is not updated.
	//}


	uint8_t bit_out = 0x00;

	if(UART_counter != TRANSMIT_FIRST_DATA)
	{
		if(UART_counter <= TRANSMIT_STOP_1)
		{
			UART_Tx_data = UART_Tx_data >> 1;     //Right shift data bits. Does not have any effect for stop bits.
		}
	}

	//If to transmit data bit, set bit_out according to UART_Tx_data.
	if(UART_counter <= TRANSMIT_LAST_DATA)
	{
		if( UART_Tx_data & 0x01 )
		{
			bit_out = 0x01;
		}
	}

	//If to transmit stop bit, set the bit to be transmitted high.
	else if(UART_counter == TRANSMIT_STOP_1)
	{
		bit_out = 0x01;
	}
	else if(UART_counter == TRANSMIT_STOP_2)
	{
		//CLEAR_UART_EXTERNAL_INTERRUPT_FLAG();
		//ENABLE_UART_EXTERNAL_INTERRUPT();
		bit_out = 0x01;
	}

	//If transmission finished. Start a new transmission if data is present in the Tx buffer. Set the UART
	//idle if not.
	else
	{
		//Check if new data is ready to be sent. If not, set UART state to idle, disable the timer interrupt and
		//enable the external interrupt to make the UART wait for new incoming data.
		if(!READ_FLAG(SW_UART_status, SW_UART_TX_BUFFER_FULL))
		{
			SET_UART_PIN();
			STOP_UART_TIMER();
			DISABLE_UART_TIMER_INTERRUPT();
			UART_counter = UART_STATE_IDLE;
			return; //Exit ISR so the counter is not updated.
		}

		//Initiate transmission if there is data in TX_buffer.
		else
		{
			UART_Tx_data = UART_Tx_buffer;
			CLEAR_FLAG( SW_UART_status, SW_UART_TX_BUFFER_FULL );
			UART_counter = TRANSMIT_FIRST_DATA - 2; //Need to substract 2 because counter is updated at the end of the ISR.
			//DISABLE_UART_EXTERNAL_INTERRUPT();
			//bit_out already set to 0x00.
		}
	}

	//Transmit bit.
	if( bit_out )
	{
		SET_UART_PIN();
	}
	else
	{
		CLEAR_UART_PIN();
	}
	UART_counter = UART_counter + 2; //Update counter.
}
